@@ -1,19 +1,18 @@
 #ifndef _HOOKS_H_
 #define _HOOKS_H_
 
-#include <glade/glade.h>
-#include "update-notifier.h"
 
 gboolean check_update_hooks(TrayApplet *un);
 void hook_tray_icon_init(TrayApplet *un);
 
-
-typedef struct _HookFileSeen HookFileSeen;
-struct _HookFileSeen
+typedef struct _HookFile HookFile;
+struct _HookFile
 {
    char *filename;
    time_t mtime;
+   char md5[16];
    gboolean cmd_run;
+   gboolean seen;
 };
 
 
@@ -21,10 +20,9 @@ typedef struct _HookDialog HookDialog;
 struct _HookDialog
 {
    GladeXML *glade_xml;
-   
+
+   // the list of all hook files
    GList *hook_files;   
-   GList *hook_files_seen;
 };
 
-
 #endif
